﻿using System;
using System.Data;
using System.Configuration;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Xml;
using System.Xml.XPath;
using System.Collections;

/// <summary>
/// Cas attribute principal, cas attributes
/// </summary>
public class DotNetCasAttributePrincipal
{
   #region Variables
    /// <summary>
    /// validate saml response
    /// </summary>
   private Boolean _status = false;
    /// <summary>
    /// user name of the identified user
    /// </summary>
   private String _userName = "";
    /// <summary>
    /// key= attribute name
    /// value= always an arraylist
    /// </summary>
   private Hashtable _attributes = new Hashtable();
    #endregion

   #region Constructor

   public DotNetCasAttributePrincipal(String xmlString)
	{
        setAttributes(xmlString);
    }
   #endregion

   #region Properties


   public Boolean isAutheniticated
    {
        get
        {
            return _status;
        }

    }

   public String userName{
       get
        {
            return _userName;
        }

    }

   public Hashtable attributes{
       get
        {
            return _attributes;
        }

   }
   #endregion

   #region Public Methods
   
    public override string ToString(){
        if (_status)
        {
            return _userName;
        }
        else
        {
            return "failed";
        }
    }
   
   #endregion

   #region Private Methods
    /// <summary>
    /// set's all attributes
    /// first it validates
    /// second the user name
    /// third the attribute values
    /// </summary>
    /// <param name="readXMl">read an xml string</param>
   private void setAttributes(String readXMl)
   {
      
       XmlDocument oXML = new XmlDocument();
       oXML.LoadXml(readXMl);
       XmlNamespaceManager nsmgr = new XmlNamespaceManager(oXML.NameTable);
       nsmgr.AddNamespace("SOAP-ENV", "http://schemas.xmlsoap.org/soap/envelope/");
       nsmgr.AddNamespace("soap", "http://schemas.xmlsoap.org/soap/envelope/");
       nsmgr.AddNamespace("", "urn:oasis:names:tc:SAML:1.0:protocol");
       nsmgr.AddNamespace("saml", "urn:oasis:names:tc:SAML:1.0:assertion");
       nsmgr.AddNamespace("samlp", "urn:oasis:names:tc:SAML:1.0:protocol");
       nsmgr.AddNamespace("xsd", "http://www.w3.org/2001/XMLSchema");
       nsmgr.AddNamespace("xsi", "http://www.w3.org/2001/XMLSchema-instance");
       XmlNodeList oList = oXML.SelectNodes("/SOAP-ENV:Envelope/SOAP-ENV:Body/*[name()='Response']/*[name()='Status']/*[name()='StatusCode']/@Value", nsmgr);
       setStatus(oXML, nsmgr);
       if (_status)
       {
           setUserName(oXML, nsmgr);
           setAttributes(oXML, nsmgr);
       }
   }


   private void setStatus(XmlDocument oXML, XmlNamespaceManager nsmgr)
   {
       XmlNode statusNode = oXML.SelectSingleNode("/SOAP-ENV:Envelope/SOAP-ENV:Body/*[name()='Response']/*[name()='Status']/*[name()='StatusCode']/@Value", nsmgr);

       if (statusNode.Value.Equals("samlp:Success"))
       {
           _status = true;
       }
   }
 
   private void setUserName(XmlDocument oXML, XmlNamespaceManager nsmgr)
   {
       XmlNode NameIdentifier = oXML.SelectSingleNode("/SOAP-ENV:Envelope/SOAP-ENV:Body/*[name()='Response']/*[name()='Assertion']/*[name()='AttributeStatement']/*[name()='Subject']/*[name()='NameIdentifier']", nsmgr);

       _userName = NameIdentifier.InnerText;

   }
   private void setAttributes(XmlDocument oXML, XmlNamespaceManager nsmgr)
   {
       XmlNodeList oList = oXML.SelectNodes("/SOAP-ENV:Envelope/SOAP-ENV:Body/*[name()='Response']/*[name()='Assertion']/*[name()='AttributeStatement']/*[name()='Attribute']", nsmgr);
       foreach (XmlNode oNode in oList)
       {
           XmlNodeList attributeList = oNode.SelectNodes("/SOAP-ENV:Envelope/SOAP-ENV:Body/*[name()='Response']/*[name()='Assertion']/*[name()='AttributeStatement']/*[name()='Attribute']/*[name()='AttributeValue']", nsmgr);
           
          
            ArrayList convertedNodes = new ArrayList();
            foreach (XmlNode aNode in oNode.ChildNodes)
               {
                   convertedNodes.Add(aNode.InnerText);
               }
               _attributes.Add(oNode.Attributes.GetNamedItem("AttributeName").Value,convertedNodes);
       }   
       }
      #endregion

   }
     

